import requests
import pandas as pd
import re
import numpy as np
from bs4 import BeautifulSoup

class scrapsociolla():

    def __init__(self, url):
        """
        url: Link web product / catalog
        pos & neg = List kalimat positif dan negatif
        """
        self.url = url 
#         self.factory = StemmerFactory() # Sastrawi
#         self.stemmer = self.factory.create_stemmer() #Sastrawi
  
    def get_url(self, url):
        """
        param :
        url: url API
        
        fungsi ini adalah untuk request ke API nya
        """
        url_api = url
        r_api = requests.get(url_api)
        return r_api
    
    def get_category(self, limit_prod=10, limit_comment=10):
        """
        Fungsi ini adalah untuk membuat dataframe komentar dari banyak produk
        """
        url_web = self.url
        category = url_web.split('/')[-1]
        url_api = f'https://catalog-api4.sociolla.com/search?limit={limit_prod}&filter=%7B%22categories.slug%22:%22{category}%22%7D'
        r_api_all = self.get_url(url_api) # Request API catalog sociolla
        if "catalog-api" in r_api_all.url:
            if r_api_all.status_code == 200:
                data_all = r_api_all.json()['data']
                category, brand_name, product_name, id = [], [], [], []
                for i in data_all:
                    try:
                        brand_name.append(i['brand']['name'])
                        product_name.append(i['name'])
                        id.append( i['id'])
                        category.append(i['default_category']['name'])
                    except:
                        break
                df_base_product = pd.DataFrame({'category':category,'brand_name': brand_name, 'product_name': product_name, 'id': id})
                df_detail_all = pd.DataFrame()
                for brand,name,id in zip(df_base_product['brand_name'], df_base_product['product_name'], df_base_product['id']):
                    other_url = f"https://soco-api.sociolla.com/reviews?skip=0&sort=-created_at&limit={limit_comment}&filter=%7B%22$and%22:[%7B%22is_published%22:true%7D,%7B%22product_id%22:{id}%7D],%22is_detail_review%22:false%7D"
                    r_api_item = self.get_url(other_url) # Request API per product dari catalog sociolla
                    if r_api_item.status_code == 200:
                        df_detail, df_skin = self.create_dataset(r_api_item, limit_comment, brand, name, id, many=True)
                        df_detail_all = pd.concat([df_detail_all, df_detail])
                        df_detail_all['id'] = df_detail_all['id'].astype('int')
                    else:
                        print(f"Product {id} Status:", r_api_item.status_code)
            else:
               raise Exception('Catalog Status :', r_api_all.status_code)
        else:
            raise Exception("Wrong Function, Maybe You Mean 'get_one()'")
        return df_base_product, df_detail_all, df_skin

    def get_search(self, limit_prod=10, limit_comment=10):
        """
        Fungsi ini adalah untuk membuat dataframe komentar dari banyak produk
        """
        keyword = self.url
        url_api = f'https://catalog-api1.sociolla.com/v3/search?filter=%7B%22keyword%22:%22{keyword}%22%7D&limit=16&skip=0'
        r_api_all = self.get_url(url_api) # Request API catalog sociolla
        if "catalog-api" in r_api_all.url:
            if r_api_all.status_code == 200:
                data_all = r_api_all.json()['data']
                category, brand_name, product_name, id = [], [], [], []
                for i in data_all:
                    try:
                        brand_name.append(i['brand']['name'])
                        product_name.append(i['name'])
                        id.append( i['id'])
                        category.append(i['default_category']['name'])
                    except:
                        break
                df_base_product = pd.DataFrame({'category':category,'brand_name': brand_name, 'product_name': product_name, 'id': id})
                df_detail_all = pd.DataFrame()
                for brand,name,id in zip(df_base_product['brand_name'], df_base_product['product_name'], df_base_product['id']):
                    other_url = f"https://soco-api.sociolla.com/reviews?skip=0&sort=-created_at&limit={limit_comment}&filter=%7B%22$and%22:[%7B%22is_published%22:true%7D,%7B%22product_id%22:{id}%7D],%22is_detail_review%22:false%7D"
                    r_api_item = self.get_url(other_url) # Request API per product dari catalog sociolla
                    if r_api_item.status_code == 200:
                        df_detail, df_skin = self.create_dataset(r_api_item, limit_comment, brand, name, id, many=True)
                        df_detail_all = pd.concat([df_detail_all, df_detail])
                        df_detail_all['id'] = df_detail_all['id'].astype('int')
                    else:
                        print(f"Product {id} Status:", r_api_item.status_code)
            else:
               raise Exception('Catalog Status :', r_api_all.status_code)
        else:
            raise Exception("Wrong Function, Maybe You Mean 'get_one()'")
        return df_base_product, df_detail_all, df_skin
    
    def get_one(self, limit=10):
        """
        Fungsi ini adalah untuk membuat dataframe komentar hanya dari satu produk
        """
        product_id = self.url
        url_api = f"https://soco-api.sociolla.com/reviews?skip=0&sort=-created_at&limit={limit}&filter=%7B%22$and%22:[%7B%22is_published%22:true%7D,%7B%22product_id%22:{product_id}%7D],%22is_detail_review%22:false%7D"
        r_api_one = self.get_url(url_api)
        if "soco-api" in r_api_one.url:
            if r_api_one.status_code == 200:
                df_detail, df_skin = self.create_dataset(r_api_one,limit=limit, many=False)
            else:
                raise Exception("STATUS :", r_api_one.status_code)
        else:
            raise Exception("Wrong Function, Maybe You Mean 'get_many()'")
        return df_detail,df_skin

    def create_dataset(self, get_data,limit, brand=None,name=None,id=None, many=True):
        """
        param:
        get_data: hasil request dari fungsi get_url()
        brand, name, id: dipakai bila menggunakan fungsi get_many() untuk memasukkan nama,id,dan brand dari produk yang berbeda2
        many: Boolean, untuk memisahkan apakah menggunakan fungi get_many() atau get_one()
        
        return: dataframe yang bersisi detail dari user(komen, umur rating, dll), dan dataframe jenis kulit dari user
        
        fungsi ini dipakai untuk membuat dataset
        """
        data = get_data.json()['data']
        details, skin_types, ages, repurchases, rating = [], [], [], [], []
        long_wear, packaging, pigmentation, texture, value_for_money = [], [], [], [], []
        ids, brands, names = [], [], []
        for i in range(limit):
            try:
                detail = data[i]['details']
                if many == True:
                    ids.append(id)
                    brands.append(brand)
                    names.append(name)
            except:
                break
            details.append(detail)
            
            try:
                repurchase = data[i]['is_repurchase']
                repurchases.append(repurchase)
            except:
                repurchases.append(np.nan)
            
            try:
                rating.append(data[i]['average_rating'])
            except:
                rating.append(np.nan)
            
            
            try:
                long_wear.append(data[i]['star_long_wear'])
#                 long_wear.append(0)
                packaging.append(data[i]['star_packaging'])
                pigmentation.append(data[i]['star_pigmentation'])
#                 pigmentation.append(0)
                texture.append(data[i]['star_texture'])
                value_for_money.append(data[i]['star_value_for_money'])
            except:
                long_wear.append(np.nan)
                packaging.append(np.nan)
                pigmentation.append(np.nan)
                texture.append(np.nan)
                value_for_money.append(np.nan)
                
            
            owner = data[i]['owner']
            try:
                skins_type = owner['skin_types']
                for i in skins_type:
                    skin_types.append(i['name'])
            except:
                skin_types.append(np.nan)
                
            try:
                ages.append(owner['age_range'])
            except:
                ages.append(np.nan)
        if many == True:
            df_detail = pd.DataFrame({'id':ids,'brand':brands,'name':names,'detail':details, 'age_range':ages, 'repurchase':repurchases, 'rating':rating,
                            'long_wear':long_wear, 'packaging':packaging, 'pigmentation':pigmentation, 'texture':texture, 'value_for_money':value_for_money})
        else:
            df_detail = pd.DataFrame({'detail':details, 'age_range':ages, 'repurchase':repurchases, 'rating':rating,
                            'long_wear':long_wear, 'packaging':packaging, 'pigmentation':pigmentation, 'texture':texture, 'value_for_money':value_for_money})
        df_skin = pd.DataFrame({'skin_type':skin_types})
        return df_detail, df_skin


import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
import re
from nltk.tokenize import sent_tokenize, word_tokenize
from Sastrawi.Stemmer.StemmerFactory import StemmerFactory
from sklearn.feature_extraction.text import TfidfVectorizer
from wordcloud import WordCloud
from supportvar import *

def clean_text(df, column):
    """
    param:
    df: dataframe yang dipilh, biasanya adalah dataframe detail
    column: kolom dari dataframe, biasanya kolom details

    return: List dari komentar yang sudah bersih

    Fungsi ini dipakai untuk membersihkan/merapihkan komentar dari para user
    """
    cleans = []
    for comment in df[column]:
        lower = comment.lower()
        preclean = re.sub(r"[0-9]\s?in\s?1", "komplit", lower)
        preclean = re.sub(r"\b(parah)\b", "", preclean)
        clean = re.sub(r"(@[A-Za-z0-9]+)|([^0-9A-Za-z \t])|(\w+:\/\/\S+)|(\d+)", "", preclean)
        clean2 = re.sub("\s+", ' ', clean).strip()
        no_stop_ind = ' '.join([token for token in word_tokenize(clean2) if token not in indonesia_s])
        no_stop_eng = ' '.join([token for token in word_tokenize(no_stop_ind) if token not in english_s])
        stem   = stemmer.stem_kalimat(no_stop_eng)
        cleans.append(stem)
    return cleans

def itterate_text(word_token):
    word_p, word_n = [], [] 
    for i,text in enumerate(word_token):
        for neg_text in neg:
            if text == neg_text:
                if word_token[i-1] == 'tidak' or word_token[i-1] == 'kurang':
                    word_p.append('tidak' + text)
                else:
                    word_n.append(text)
        for pos_text in pos:
            if text == pos_text:
                if word_token[i-1] == 'tidak' or word_token[i-1] == 'kurang':
                    word_n.append('tidak' + text)
                else:
                    word_p.append(text)
    return word_p, word_n

def comment_sentiment(clean_text, df_ref, treshold = 2):
    """
    param:
    clean_text: teks yang sudah dibersihkan dalam bentuk list
    df_ref: dataframe dan kolom yang merupakan asal dari teks yang sudah dibersihkan, cth:df_detail['details']

    return: dataframe komentar asli(yg belum dibersihkan) dan hasil sentimentnya

    Fungsi ini dipakai untuk memberikan nilai sentimen dari komentar user
    """
    word_p, word_n, sentiment = [], [], []
    for comment,rating in zip(clean_text,df_ref['rating']):
#             count_p = 0 #nilai positif
        count_n = 0 #nilai negatif
        word_token = word_tokenize(comment)
        if rating >= 4.0:
            sentiment.append('Positive')
            word_p2, word_n2 = itterate_text(word_token)
            word_p.extend(word_p2)
            word_n.extend(word_n2)
        elif (rating >= 3.0) and (rating<=3.99): 
            for i,text in enumerate(word_token):
                for neg_text in neg:
                    if text == neg_text:
                        if word_token[i-1] == 'tidak' or word_token[i-1] == 'kurang':
                            count_n += 0
                            word_p.append('tidak' + text)
                        if word_token[i-1] == 'mudah' or word_token[i-1] == 'gampang':
                            if word_token[i-2] == 'tidak':
                                count_n += 0
                                word_p.append('tidakmudah' + text)
                            else:
                                word_n.append('mudah'+text)
                                count_n += 1
                        else:
                            word_n.append(text)
                            count_n += 1
                for pos_text in pos:
                    if text == pos_text:
                        if word_token[i-1] == 'tidak' or word_token[i-1] == 'kurang':
                            word_n.append('tidak' + text)
                            count_n += 1
                        else:
                            word_p.append(text)

            if count_n > treshold:
                sentiment.append('Negative')
            else:
                sentiment.append('Positive')
        elif rating < 3.0:
            sentiment.append('Negative')
            word_p2, word_n2 = itterate_text(word_token)
            word_p.extend(word_p2)
            word_n.extend(word_n2)
    df_sentiment = pd.DataFrame({'comment': df_ref['detail'].tolist(),'clean_comment':clean_text, 'sentiment':sentiment})
    return df_sentiment, word_n, word_p

def rating(df):
    """
    param:
    df: dataframe yang dipilih untuk dihitung ratingnya


    fungsi ini dipakai untuk menghitung rating dari suatu produk
    """
    overall_rating = round(df['rating'].mean(),2)
    long_wear = round(df['long_wear'].mean(),2)
    packaging = round(df['packaging'].mean(),2)
    pigmentation = round(df['pigmentation'].mean(),2)
    texture = round(df['texture'].mean(),2)
    value_for_money = round(df['value_for_money'].mean(),2)
    return pd.DataFrame({'ovr_rating':overall_rating, 'long_wear':long_wear, 'packaging':packaging, 'pigmentation':pigmentation, 'texture':texture, 'value_for_money':value_for_money}, index=['Rating']).T

def plot_top10_skin(df, column, top=10):
    """
    param:
    df: dataframe yang dipilh, biasanya adalah dataframe skin
    column: kolom dari dataframe
    top: jumlah data yang akan ditampilkan

    fungsi ini dipakai untuk membuat bar_plot dari jenis kulit user
    """
    top10 = df[column].value_counts().iloc[:top].index
    sns.countplot(data=df, y=column, order=top10)
    return plt.show()

def plot_repurchase(df, column):
    labels = df.groupby(column)[column].count().sort_values(ascending=False).reset_index(name='count')[column].tolist()
    fig= plt.figure()
    fig.patch.set_facecolor('white')
    plt.pie(df[column].value_counts(), autopct='%.2f%%', labels=labels)
    plt.show()

def barPerc_without_hue_h(ax, feature):
    total = sum(feature)
    for p in ax.patches:
        percentage = '{:.1f}%'.format(100 * p.get_height()/total)
        x = p.get_x() + p.get_width() - 0.5
        y = p.get_y() + p.get_height()
        ax.annotate(percentage, (x, y), size = 12)

def plot_age(df, column):
    groupbyx = df.groupby('age_range')['age_range'].count().reset_index(name='count')
    sum1 = groupbyx['count'].values
    ax = sns.barplot(data=groupbyx, x='age_range', y='count')
    barPerc_without_hue_h(ax, sum1)
    plt.show()


def wordcloud(word_n, word_p, top_n=10, top_p=10):
    """
    param:
    word_n: list kalimat negatif yang ada di komentar
    word_p: list kalimat positif yang ada di komentar
    top_n: jumlah kalimat negatif yang ingin ditampilkan
    top_p: jumlah kalimat positif yang ingin ditampilkan
    """
    count_word_n = pd.DataFrame({'word':word_n}).groupby('word')['word'].count().sort_values(ascending=False).head(top_n).reset_index(name='count')
    count_word_p = pd.DataFrame({'word':word_p}).groupby('word')['word'].count().sort_values(ascending=False).head(top_p).reset_index(name='count')
    join_p = ' '.join(count_word_p['word'])
    join_n = ' '.join(count_word_n['word'])
    wordcloud_p = WordCloud(width=1600, height=800, max_font_size=200,max_words=top_n, background_color="white").generate(join_p)
    wordcloud_n = WordCloud(width=1600, height=800, max_font_size=200,max_words=top_p, background_color="white").generate(join_n)
    fig, axs = plt.subplots(1,2, figsize=(30,30))
    axs[0].imshow(wordcloud_n, interpolation='bilinear')
    axs[0].axis('off')
    axs[0].set_title('Negative Word')
    axs[1].imshow(wordcloud_p, interpolation='bilinear')
    axs[1].axis('off')
    axs[1].set_title('Positive Word')
    return plt.show()

def sentiment_summary(df, column):
    labels = df.groupby(column)[column].count().sort_values(ascending=False).reset_index(name='count')[column].tolist()
    fig= plt.figure()
    fig.patch.set_facecolor('white')
    plt.pie(df[column].value_counts(), autopct='%.2f%%', labels=labels)
    plt.show()

    print("Positive Comment:")
    try:
        print(df[df[column]=='Positive']['comment'].sample(1).tolist(), '\n')
    except:
        print('Info : Tidak ada comment Positive')
    print("Negative Comment:")
    try:
        print(df[df[column]=='Negative']['comment'].sample(1).tolist(), '\n')
    except:
        print('Info : Tidak ada comment Negative')
